unit ListModelPresenter;

interface

uses
  ListModel, ListBoxView, ComboBoxView, Controls;

type

  TListModelPresenter = class
  private
    fListModel: IListModel;
    fListBox: TListBoxView;
    fComboBox: TComboBoxView;
  public
    constructor Create(Model: IListModel); virtual;
    procedure HandleComboSelect(Sender: TObject);
    procedure HandleDragDrop(Sender, Source: TObject;
                             X, Y: Integer);
    procedure HandleDragOver(Sender, Source: TObject;
                             X, Y: Integer;
                             State: TDragState;
                             var Accept: Boolean);
    procedure HandleFormCreate(Sender: TObject);
  end;

implementation

uses
  StdCtrls, Types, Forms;

{ TListModelPresenter }

constructor TListModelPresenter.Create(Model: IListModel);
begin
  inherited Create;
  fListModel := Model;
end;

procedure TListModelPresenter.HandleComboSelect(Sender: TObject);
var
  cb: TComboBox;
begin
  cb := Sender as TComboBox;
  fListModel.Move(cb.Items[cb.ItemIndex], fListModel.Item[0]);
end;

procedure TListModelPresenter.HandleDragDrop(Sender, Source: TObject;
                                             X, Y: Integer);
var
  SourceList, SenderList: TListBox;
  pt: TPoint;
  idx: Integer;
  tempStr: string;
begin
  SenderList := Sender as TListBox;
  SourceList := Source as TListBox;
  pt.X := X;
  pt.Y := Y;

  idx := SenderList.ItemAtPos(pt, False);

  if idx = SourceList.Count then
  begin
    tempStr := fListModel.Item[SourceList.ItemIndex];
    fListModel.BeginUpdate;
    fListModel.Remove(tempStr);
    fListModel.Add(tempStr);
    fListModel.EndUpdate;
  end
  else
    fListModel.Move(SourceList.Items[SourceList.ItemIndex], SenderList.Items[idx]);
end;

procedure TListModelPresenter.HandleDragOver(Sender, Source: TObject;
                                             X, Y: Integer;
                                             State: TDragState;
                                             var Accept: Boolean);
var
  SourceList, SenderList: TListBox;
  pt: TPoint;
  idx: Integer;
begin
  SenderList := Sender as TListBox;
  SourceList := Source as TListBox;
  pt.X := X;
  pt.Y := Y;

  idx := (SenderList).ItemAtPos(pt, False);
  if idx < 0 then
    idx := 0;
  if idx >= SenderList.Count then
    idx := Pred(SenderList.Count);

  Accept := fListModel.Item[idx] <> fListModel.Item[(SourceList).ItemIndex];
end;

procedure TListModelPresenter.HandleFormCreate(Sender: TObject);
var
  Form: TForm;
begin
  Form := Sender as TForm;

  fListBox := TListBoxView.Create(Form);
  fListBox.Parent := Form;
  fListBox.Height := 300;
  fListBox.Width := 200;
  fListBox.Font.Size := 18;
  fListBox.DragMode := dmAutomatic;
  fListBox.OnDragOver := HandleDragOver;
  fListBox.OnDragDrop := HandleDragDrop;
  (fListModel as ISubject).AttachObserver(fListBox);

  fComboBox := TComboBoxView.Create(Form);
  fComboBox.Parent := Form;
  fComboBox.Top := 50;
  fComboBox.Left := 250;
  fComboBox.Width := 200;
  fComboBox.Font.Size := 18;
  fComboBox.OnSelect := HandleComboSelect;
  (fListModel as ISubject).AttachObserver(fComboBox);
end;

end.
